import Flutter
import UIKit
import AppConsentKit
import AppConsentUIKit

public class SwiftAppconsentClassicPlugin: NSObject, FlutterPlugin {
  var ackit: ACUIKit?

  override init() {
    super.init()
  }


  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "appconsent_classic", binaryMessenger: registrar.messenger())
    let instance = SwiftAppconsentClassicPlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    switch call.method {
    case "setup":
      setup(call, result: result)
    case "presentNotice":
      presentNotice(call, result: result)
    case "clearConsent":
      clearConsent(result: result)
    case "checkForUpdate":
      checkForUpdate(result: result)
    case "isSubjectToGDPR":
      isSubjectToGDPR(result: result)
    case "consentGiven":
      consentGiven(result: result)
    case "consentableAllowed":
      consentableAllowed(call, result: result)
    case "appTrackingIsAvailable":
      appTrackingIsAvailable(result: result)
    case "appTrackingAuthorizationGiven":
      appTrackingAuthorizationGiven(result: result)
    case "appTrackingAuthorizationStatus":
      appTrackingAuthorizationStatus(result: result)
    default:
      result(FlutterMethodNotImplemented)
    }
  }
  
  func setup(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    guard let args = call.arguments as? Dictionary<String, Any> else {
      result(nil)
      return
    }
    guard let appKey = args["appKey"] as? String else {
      result(nil)
      return
    }
    guard let forceApplyGDPR = args["forceApplyGDPR"] as? Bool else {
      result(nil)
      return
    }
    guard let forceATT = args["forceATT"] as? Bool else {
      result(nil)
      return
    }
    
    self.ackit = ACUIKit(withAppKey: appKey, forceApplyGDPR: forceApplyGDPR, forceATT: forceATT)
    result(nil)
  }

  func presentNotice(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    guard let args = call.arguments as? Dictionary<String, Any> else {
      result(nil)
      return
    }
    guard let force = args["force"] as? Bool else {
      result(nil)
      return
    }
    let viewController: UIViewController = (UIApplication.shared.delegate?.window??.rootViewController)!
    ackit!.presentNotice(force: force, viewController: viewController)
    result(nil)
  }

  func clearConsent(result: @escaping FlutterResult) {
    ackit!.clearConsent()
    result(nil)
  }

  func checkForUpdate(result: @escaping FlutterResult) {
    ackit!.checkForUpdate({ success in
      result(success)
    }, failure: { error in
      result(false)
    })
  }

  func isSubjectToGDPR(result: @escaping FlutterResult) {
    let isGDPRCountry = ackit!.isGDPRCountry()
    result(isGDPRCountry)
  }

  func consentGiven(result: @escaping FlutterResult) {
    let consentGiven = ackit!.consentGiven()
    result(consentGiven)
  }

  func consentableAllowed(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    guard let args = call.arguments as? Dictionary<String, Any> else {
      result(nil)
      return
    }
    guard let iabId = args["iabId"] as? UInt32 else {
      result(nil)
      return
    }
    guard let type = args["type"] as? Int else {
      result(nil)
      return
    }
    guard let consentableType = ACConsentableType(rawValue: type) else {
      result(nil)
      return
    }
    
    let allowed = ackit!.consentableAllowed(iabId: iabId, type: consentableType)
    result(allowed)
  }

  func appTrackingIsAvailable(result: @escaping FlutterResult) {
    let appTrackingIsAvailable = ackit!.appTrackingIsAvailable()
    result(appTrackingIsAvailable)
  }

  func appTrackingAuthorizationGiven(result: @escaping FlutterResult) {
    let appTrackingAuthorizationGiven = ackit!.appTrackingAuthorizationGiven()
    result(appTrackingAuthorizationGiven)
  }

  func appTrackingAuthorizationStatus(result: @escaping FlutterResult) {
    let appTrackingAuthorizationStatus = ackit!.appTrackingAuthorizationStatus()
    result(appTrackingAuthorizationStatus)
  }
}
